@@ -34,4 +34,9 @@ def import_remote_dump(kwargs):
                            stdin=zcat.stdout)
   zcat.stdout.close()
 
-  sys.exit(mysql.poll())
+  returncode = mysql.poll()
+
+  if returncode == 0:
+    open(lock_file, 'w').close() # Just a touch
+
+  sys.exit(returncode)
